/* version du 7/11/02 basee sur picachof */
/* copyright A. GLEYZES aeropic 2002 */
#include <pic.h>

// variables globales
// ******************
unsigned char COUNTER, PC_OFFSET, EEBYTE, EEDATA, EEADDR;

// define locaux
// *************
#define WR_EE 1         
#define RD_EE 0
#define BOUGIE GP0
#define bougie2 0
#define bougie1 1
#define PLUS GP1
#define MOINS GP3
#define ON 0
#define S_HORL GP2
#define S_CHAUFFE GP4
#define S_ECH GP5
#define chauffe_max 30 // correspond a 15 % de rapport cyclique
#define chauffe_min 4

__CONFIG(INTRC | UNPROTECT); 	/* rq MCLRE et XDTE sont a 0 */

/*******************************************************
*
*	electronique de gestion de chauffe bougie (glow driver)
*
*	p7 : GP0 = entree bougie1/bougie2 
*	p6 : GP1 = entree PLUS: augmente la chauffe 
*	p5 : GP2 = sortie horloge 4000 Hz (N/U)
*	p4 : GP3 = entree MOINS : diminue la chauffe 
*	p3 : GP4 = sortie chauffe vers MOSFET
*	p2 : GP5 = sortie echantillonage chauffe idem GP4
*
*	GP0, 1 et 3 : pull up internes actifs
*
*	Les sorties GP4 et 5 presentent un signal rectangulaire de frequence 40 Hz .
*	Le rapport cyclique sur GP4&5 est reglable entre 2% et une valeur max (typ 10%).
*	
*	GP0 sert a selectionner la configuration bougie1 ou bougie2. 
*     Le systeme permet donc de gerer deux types de bougies necessitant des tensions de chauffe
*     differentes.
*
*	Au premier bootstrap, le pourcentage de chauffe est  2%, la valeur peut-etre 
*	augmentee (resp. diminuee) en appuyant sur les boutons PLUS (resp. MOINS)
* 	Par dfaut la chauffe maxi est limitee a 15%, on peut memoriser une valeur
*	maxi de chauffe differente (plus faible) apres avoir augmente la chauffe 
*	jusqu'a la valeur souhaitee, en appuyant sur les 2 BP simultanement.
*	On peut revenir a la valeur maxi de 10% en tenant le BP PLUS enfonce au bootstrap
*	NOTA : la valeur max est specifique a chacune des deux bougies. 
*            Il faut donc rgler chaque bougie indpendamment en jouant sur l'inter.
*
*     A la premiere utilisation, il est conseill de remplacer la bougie 
*     par une ampoule de voiture 15W, de verifier le montage en jouant sur l'intensit 
*     de la lampe (boutons PLUS et MOINS. On doit voir l'ampoule passer de quasi eteinte a
*     a peine brillante. Revenir a la plus faible lumiere filament a peine rouge.
*     On peut alors brancher une bougie.
*     On augmente progressivement la temperature du filament de la bougie (bouton PLUS) jusqu'a
*     obtenir un rouge profond.
*
*	La derniere valeur de chauffe utilisee pour chaque bougie (une valeur par bougie)
*	est memorisee pour etre selectionnee au bootstrap ...
*
***********************************************************/




// ATTENTION : la fonction a12c5 doit etre mappe dans les adresses basses
// =======================================================================

fastcall unsigned char a12c5_eeprom(unsigned char op, unsigned char *p_dat) 
{
	unsigned char i;
   
   if (op==WR_EE)
   {
      EEDATA=*p_dat;
	
#asm

    GOTO WRITE_BYTE
#endasm
i = 2;			// pour contourner un bug du compilo
   }
i=3;			// pour contourner un bug compilo
#asm
   GOTO READ_RANDOM


READ_CURRENT
    MOVLW   0x84    // PC offset for read current addr.  EE_OK bit7='1'
    MOVWF   _PC_OFFSET    // Load PC offset
    GOTO    INIT_READ_CONTROL

WRITE_BYTE
    MOVLW   0x80   // PC offset for write byte.  EE_OK: bit7 = '1'
    GOTO    INIT_WRITE_CONTROL

	
READ_RANDOM  
    MOVLW   0x83   // PC offset for read random.  EE_OK: bit7 = '1'

INIT_WRITE_CONTROL
    MOVWF   _PC_OFFSET     // Load PC offset register, value preset in W
    MOVLW   0xA0
    // Control byte with write bit, bit 0 = '0'

START_BIT
    BCF     _GPIO,0x6  // Start bit, SDA and SCL preset to '1'

//******* Set up output data (control, address, or data) and counter 
//*****
PREP_TRANSFER_BYTE
    MOVWF   _EEBYTE        // Byte to transfer to EEPROM already in W
    MOVLW   8            // Counter to transfer 8 bits
    MOVWF   _COUNTER

//************  Clock out data (control, address, or data) byte  
//******
OUTPUT_BYTE
        BCF     _GPIO, 0x7  // Set clock low during data set-up
        RLF     _EEBYTE, F  // Rotate left, high order bit into carry bit
        BCF     _GPIO, 0x06  // Set data low, if rotated carry bit is
        BTFSC   _STATUS, 0x0  //   a '1', then:
        BSF     _GPIO, 0x06  // reset data pin to a one, otherwise leave low
        NOP
        BSF     _GPIO, 0x07  // clock data into EEPROM
        DECFSZ  _COUNTER, F // Repeat until entire byte is sent
        GOTO    OUTPUT_BYTE
        NOP                // Needed to meet Timing (Thigh=4000nS)

//**********************  Acknowkedge Check 
//*******
        BCF     _GPIO, 0x07  // Set SCL low, 0.5us < ack valid < 3us
        NOP                // Needed to meet Timing (Tlow= 4700nS)
        BSF     _GPIO, 0x06
        NOP
        NOP
        NOP                // Necessary for SCL Tlow at low voltage,
        NOP                // Tlow=4700nS
        BSF     _GPIO, 0x07  // Raise SCL, EEPROM acknowledge still valid
        BTFSC   _GPIO, 0x06  // Check SDA for acknowledge (low)
        BCF     _PC_OFFSET, 7 // If SDA not low (no ack), set error flag
        BCF     _GPIO, 0x07  // Lower SCL, EEPROM release bus

        BTFSS   _PC_OFFSET, 7 // If no error continue, else stop bit
        GOTO    STOP_BIT

//*****  Set up program counter offset, based on EEPROM operating mode  
//*****
        MOVF    _PC_OFFSET, W
        ANDLW   0x0F
        ADDWF   _PCL, F
        GOTO    INIT_ADDRESS
        // PC offset=0, write control done, send address
        GOTO    INIT_WRITE_DATA
        // PC offset=1, write address done, send data
        GOTO    STOP_BIT
        // PC offset=2, write done, send stop bit
        GOTO    INIT_ADDRESS
        // PC offset=3, write control done, send address
        GOTO    INIT_READ_CONTROL
        // PC offset=4, send read control
        GOTO    READ_BIT_COUNTER
        // PC offset=5, set counter and read byte
        GOTO    STOP_BIT
        // PC offset=6, random read done, send stop
//**********  Initalize EEPROM data (address, data, or control) bytes  
//******
INIT_ADDRESS
    INCF    _PC_OFFSET, F // Increment PC offset to 2 (write) or to 4 (read)
    MOVF    _EEADDR,W     // Put EEPROM address in W, ready to send to EEPROM
    GOTO    PREP_TRANSFER_BYTE

INIT_WRITE_DATA
    INCF    _PC_OFFSET, F // Increment PC offset to go to STOP_BIT next
    MOVF    _EEDATA,W     // Put EEPROM data in W, ready to send to EEPROM
    GOTO    PREP_TRANSFER_BYTE

INIT_READ_CONTROL
    BSF     _GPIO, 0x07 // Raise SCL
    BSF     _GPIO, 0x06 // raise SDA
    INCF    _PC_OFFSET, F // Inc PC offset to go to READ_BIT_COUNTER next
    MOVLW   0xA1  // Set up read control byte, ready to send to EEPROM
    GOTO    START_BIT    //   bit 0 = '1' for read operation

//**************************  Read EEPROM data  
//*****
READ_BIT_COUNTER
    BSF     _GPIO, 0x06 // set data bit to 1 so we're not pulling bus down.
    NOP
    BSF     _GPIO, 0x07
    MOVLW   8           // Set counter so 8 bits will be read into EEDATA
    MOVWF   _COUNTER

READ_BYTE
    BSF     _GPIO, 0x07 // Raise SCL, SDA valid.  SDA still input from ack
    BSF     _STATUS, 0x0              // Assume bit to be read = 1
    BTFSS   _GPIO, 0x06 // Check if SDA = 1
    BCF     _STATUS, 0x0         // if SDA not = 1 then clear carry bit
    RLF     _EEDATA, F // rotate carry bit (=SDA) into EEDATA;
    BCF     _GPIO, 0x07 // Lower SCL
    BSF     _GPIO, 0x06 // reset SDA
    DECFSZ  _COUNTER, F //Decrement counter
    GOTO    READ_BYTE  // Read next bit if not finished reading byte

    BSF     _GPIO, 0x07
    NOP
    BCF     _GPIO, 0x07
//******************  Generate a STOP bit and RETURN  ***********************
//*****
STOP_BIT
    BCF     _GPIO, 0x06 // SDA=0, on TRIS, to prepare for transition to '1'
    BSF     _GPIO, 0x07 // SCL = 1 to prepare for STOP bit
    NOP       // equivalent 4 NOPs neccessary for I2C spec Tsu:sto = 4.7us
    NOP
    NOP
    NOP
    BSF     _GPIO, 0x06 // Stop bit, SDA transition to '1' while SCL high

#endasm
    *p_dat = EEDATA;
    return(PC_OFFSET);  // 1 is OK, 0 is NO
}

/***************************************************************
*
*	electronique de gestion de chauffe bougie (glow driver)
*                      programme principal
*
/***************************************************************/

main (void)
{
	unsigned char speed ;	/* gestion horloge timer */
	unsigned char phase;	/* compteur de sous cycle pour frequence Ftimer/100 */
				      /* vaut entre 0 et 199 */
	unsigned char rapport1;	/* valeur du pourcentage de chauffe bougie 1 */
	unsigned char rapport2;	/* valeur du pourcentage de chauffe bougie 2 */

	unsigned char i;   	/* filtre anti rebond */
	unsigned char max;	/* valeur max de chauffe bougie 1*/
	unsigned char max2;	/* valeur max de chauffe bougie 2*/

	unsigned char tempo, res;

/* INITIALISATIONS */
/* =============== */
	OPTION = GPWU | !GPPU | T0SE | !PSA | 0x01 ; /* wake up on pin change
							Pull up enabled
							Internal cycle clock
							High to low source edge
							Prescaler assigned to TMR0 
							rate 1:4 */
	TRIS = 0b001011 ; 			/* GP2, 4 et 5  en sortie */
	GPIO = 0x0;				      /* sorties a 0 */
	i = 0;
	phase = 0;

	speed = 228;		           // 228 est la valeur du timer pour 4000 Hz

	
// premiere lecture a blanc pour init lignes SDA SLC
	EEADDR = 0x02;                     // max chauffe stockee a l'adresse 2 de l'EEPROM
      res = a12c5_eeprom(RD_EE, &max);

/* initialisation de la valeur max de chauffe */
// ==========================================

	EEADDR = 0x02;          // max chauffe stockee a l'adresse 2 de l'EEPROM
      res = a12c5_eeprom(RD_EE, &max);
	if ((max < 5)||(max > chauffe_max)) max = chauffe_max;    // filtre EEPROM non initialisee

      EEADDR = 0x03;          // max chauffe bougie 2 stockee a l'adresse 3 de l'EEPROM
      res = a12c5_eeprom(RD_EE, &max2);
	if ((max2 < 5)||(max2 > chauffe_max)) max2 = chauffe_max;    // filtre EEPROM non initialisee



/* lecture valeur chauffe bougie1  */
// ==============================
	EEADDR = 0x00;          //  rapport chauffe bougie1 stockee a l'adresse 0 de l'EEPROM
      res = a12c5_eeprom(RD_EE, &rapport1);
	if (rapport1 > max) rapport1 = chauffe_min;    // filtre EEPROM non initialisee




/* lecture valeur chauffe bougie2  */
/* =============================== */

	EEADDR = 0x01;          // rapport chauffe bougie2 stockee a l'adresse 1 de l'EEPROM
	
      res = a12c5_eeprom(RD_EE, &rapport2);
	if (rapport2 > max2) rapport2 = chauffe_min;    // filtre EEPROM non initialisee



// resets au boostrap si BP PLUS appuye
// ====================================
	
	if (BOUGIE == bougie1)		// on est sur bougie 1
	{
	  if (PLUS == ON) 		// reset valeur max
	  {
		  max = chauffe_max;
		  EEADDR = 0x02;			// stocke a l'adresse 2
		  tempo = max;
	   	  res = a12c5_eeprom(WR_EE, &tempo);	 // sauvegarde ds l'EEPROM
	  }
	}
	else // on est sur bougie 2
	{
	  if (PLUS == ON) 		// reset valeur max
	  {
		  max2 = chauffe_max;
		  EEADDR = 0x03;			// stocke a l'adresse 3
		  tempo = max2;
	   	  res = a12c5_eeprom(WR_EE, &tempo);	 // sauvegarde ds l'EEPROM
	  }
	}


	
		
/* GESTION DES HORLOGES */
/* ==================== */
horloge :
	TMR0 = speed; 
	if (BOUGIE == bougie2) 		// inter ferme = bougie 2
	{
	phase++;          /* incremente phase de 0 a 200 a chaque demi periode 4000 Hz */
		if (phase < rapport2) 
		{
			S_CHAUFFE = 1;
			S_ECH = 1;  /* gestion sortie chauffe */
		}
		else 
		{
			S_CHAUFFE = 0 ;
			S_ECH = 0;
		}
		if (phase >= 200)		/* gestion des boutons */
		{
			phase = 0;
			if (PLUS == ON) 
			{
				if (i==0)		/* on peut valider */
				{
				  if (MOINS == ON) goto saut;
				  rapport2++ ; /* incremente rapport cyclique */
				  if (rapport2 > max2) rapport2 = max2;
				  EEADDR = 0x01;
				  tempo = rapport2;
	   	 		  res = a12c5_eeprom(WR_EE, &tempo);	 // sauvegarde ds l'EEPROM
				  goto saut;
				}
				else if (i == 3)
				{
				  if (MOINS == ON)  // les 2 BP sont ON ensemble
						  // ==> memoriser valeur max chauffe
				  {
				    max2 = rapport2;
				    EEADDR = 0x03;
				    tempo = max2;
	   	 		    res = a12c5_eeprom(WR_EE, &tempo);	 // sauvegarde ds l'EEPROM
				    goto saut;
				  }
				}
			}
 			if (MOINS == ON) 
			{
				if (i==0)		/* on peut valider */
				{
				  rapport2-- ; /* decremente rapport cyclique */
				  if (rapport2 == chauffe_min-1) rapport2 = chauffe_min;
				  EEADDR = 0x01;
	   	 		  tempo = rapport2;
	   	 		  res = a12c5_eeprom(WR_EE, &tempo);	 // sauvegarde ds l'EEPROM
				}
			}
saut :
			i++;				/* filtre  environ 0,37 sec */
			if (i==15) i=0;
		}
	}

	else			// inter ouvert = bougie 1
	{
					
		phase++;          /* incremente phase de 0 a 200*/
		if (phase < rapport1) 
		{
			S_CHAUFFE = 1;
			S_ECH = 1;  /* gestion sortie chauffe */
		}
		else 
		{
			S_CHAUFFE = 0 ;
			S_ECH = 0;
		}
		if (phase >= 200)		/* gestion des boutons */
		{
			phase = 0;
			if (PLUS == ON) 
			{
				if (i==0)		/* on peut valider */
				{
				  if (MOINS == ON) goto saut;
				  rapport1++ ; /* incremente rapport cyclique */
				  if (rapport1 > max) rapport1 = max;
				  EEADDR = 0x00;
				  tempo = rapport1;
	   	 		  res = a12c5_eeprom(WR_EE, &tempo);	 // sauvegarde ds l'EEPROM
				  goto saut2;
				}
				else if (i == 3)
				{
				  if (MOINS == ON)  // les 2 BP sont ON ensemble
						  // ==> memoriser valeur max chauffe
				  {
				    max = rapport1;
				    EEADDR = 0x02;
				    tempo = max;
	   	 		    res = a12c5_eeprom(WR_EE, &tempo);	 // sauvegarde ds l'EEPROM
				    goto saut2;
				  }
				}
			}
 			if (MOINS == ON) 
			{
				if (i==0)		/* on peut valider */
				{
				  rapport1-- ; /* decremente rapport cyclique */
				  if (rapport1 == chauffe_min-1) rapport1 = chauffe_min;
				  EEADDR = 0x00;
	   	 		  tempo = rapport1;
	   	 		  res = a12c5_eeprom(WR_EE, &tempo);	 // sauvegarde ds l'EEPROM
				}
			}
saut2 :
			i++;				/* filtre  environ 0,37 sec */
			if (i==15) i=0;
		}
	}
	do {			/* attente fin du timer de 250 - 6 usec */
	} while (TMR0);
	S_HORL = !S_HORL;
	asm("NOP");
goto horloge ;
}
