// version du 3/11/05   : test OK : attention au pull up GP3...
// V2.0
//  
// PIC 12F675
// 
// ATTENTION : sur les 675 la pin GPIO3 n'a pas de pull up interne 
//             ==> prvoir une rsistance !

// V1 = version du 7/11/02 basee sur picachof */
/* copyright A. GLEYZES aeropic 2002 */
#include <pic.h>

__CONFIG(UNPROTECT & BORDIS & MCLRDIS & PWRTEN & WDTDIS & INTIO); // config pour 12F675
		/* 	Data et Pgm non protgs
			BrownOut-Rst disabled
			MmCLR disabled => GPIO3 : Port-In
			Power up timer enabled
			Watchdog disable
			Intern.Osc, GPIO4-5 : Port I-O */

// variables globales
// ******************
unsigned char COUNTER, PC_OFFSET ;

// define locaux
// *************

#define BOUGIE GPIO0
#define bougie2 0
#define bougie1 1
#define PLUS GPIO1
#define MOINS GPIO3
#define ON 0
#define S_HORL GPIO2
#define S_CHAUFFE GPIO4
#define S_ECH GPIO5
#define chauffe_max 30 // correspond a 15 % de rapport cyclique
#define chauffe_min 4



/*******************************************************
*
*	electronique de gestion de chauffe bougie (glow driver)
*
*	p7 : GPIO0 = entree bougie1/bougie2 
*	p6 : GPIO1 = entree PLUS: augmente la chauffe 
*	p5 : GPIO2 = sortie horloge 4000 Hz (N/U)
*	p4 : GPIO3 = entree MOINS : diminue la chauffe 
*	p3 : GPIO4 = sortie chauffe vers MOSFET
*	p2 : GPIO5 = sortie echantillonage chauffe idem GPIO4
*
*	GPIO0, 1 et 3 : pull up internes actifs (675 rajouter un pull up externe)
*
*	Les sorties GPIO4 et 5 presentent un signal rectangulaire de frequence 40 Hz .
*	Le rapport cyclique sur GPIO4&5 est reglable entre 2% et une valeur max (typ 10%).
*	
*	GPIO0 sert a selectionner la configuration bougie1 ou bougie2. 
*     Le systeme permet donc de gerer deux types de bougies necessitant des tensions de chauffe
*     differentes.
*
*	Au premier bootstrap, le pourcentage de chauffe est  2%, la valeur peut-etre 
*	augmentee (resp. diminuee) en appuyant sur les boutons PLUS (resp. MOINS)
* 	Par dfaut la chauffe maxi est limitee a 15%, on peut memoriser une valeur
*	maxi de chauffe differente (plus faible) apres avoir augmente la chauffe 
*	jusqu'a la valeur souhaitee, en appuyant sur les 2 BP simultanement.
*	On peut revenir a la valeur maxi de 10% en tenant le BP PLUS enfonce au bootstrap
*	NOTA : la valeur max est specifique a chacune des deux bougies. 
*            Il faut donc rgler chaque bougie indpendamment en jouant sur l'inter.
*
*     A la premiere utilisation, il est conseill de remplacer la bougie 
*     par une ampoule de voiture 15W, de verifier le montage en jouant sur l'intensit 
*     de la lampe (boutons PLUS et MOINS. On doit voir l'ampoule passer de quasi eteinte a
*     a peine brillante. Revenir a la plus faible lumiere filament a peine rouge.
*     On peut alors brancher une bougie.
*     On augmente progressivement la temperature du filament de la bougie (bouton PLUS) jusqu'a
*     obtenir un rouge profond.
*
*	La derniere valeur de chauffe utilisee pour chaque bougie (une valeur par bougie)
*	est memorisee pour etre selectionnee au bootstrap ...
*
***********************************************************/

//function prototypes
void interrupt isr(void);

// copyright en EEPROM
#asm	// Ecriture en EEPROM :
	psect	eedata,delta=2,abs,ovrld
	org		2100h
	db		4,4,30,30,0,0,0,0					// chauffe BG1, chauffe BG2, maxBG, max BG2
	db		'A','E','R','O','P','I','C',' '
	db		'G','L','O','W','6','7','5',' '
	db		'V','2','.','0',' ','(','C',')'
	db		'2','0','0','5',' ','(','C',')'
	
			

	// Ecriture de la valeur OSCCAL en Flash :
	org		3FFh			// sert pour le debug MPLAB, A ENLEVER EN FINAL !!!
	dw		349Ch

#endasm

/********************************************************/
/*      routine de gestion de l'EEPROM                 */
/******************************************************/
void eeprom_write(unsigned char addr, unsigned char value)
{
	EEPROM_WRITE(addr, value);
	GIE=0;  // si besoin	
}	

unsigned char eeprom_read(unsigned char addr)
{
    	return(EEPROM_READ(addr)); 
}	




/***************************************************************
*
*	electronique de gestion de chauffe bougie (glow driver)
*                      programme principal
*
/***************************************************************/

main (void)
{
	unsigned char speed ;	/* gestion horloge timer */
	unsigned char phase;	/* compteur de sous cycle pour frequence Ftimer/100 */
				      /* vaut entre 0 et 199 */
	unsigned char rapport1;	/* valeur du pourcentage de chauffe bougie 1 */
	unsigned char rapport2;	/* valeur du pourcentage de chauffe bougie 2 */

	unsigned char i;   	/* filtre anti rebond */
	unsigned char max;	/* valeur max de chauffe bougie 1*/
	unsigned char max2;	/* valeur max de chauffe bougie 2*/

	unsigned char tempo;

/* INITIALISATIONS */
/* =============== */

	
	OSCCAL = _READ_OSCCAL_DATA();
	TRISIO	= 0b001011 ; 			/* GPIO2, 4 et 5  en sortie */
	CMCON = 0b00000111;     //init du comparateur : general IO
	ANSEL = 0x00;           //turn off analog inputs
	ADON = 0x00;            //turn off A/D
	
	
							
	GPIO = 0x00;            //reset GPIO
	// OPTION = GPPU | !T0CS | T0SE | !PSA | 0x01; attention cette ligne ne passe pas
	OPTION = 0b00000001;						// oblig de passer au binaire !
		 /* Pull up enabled
			Internal cycle clock
			High to low source edge
			Prescaler assigned to timer
			Prescaler 1:4 */
	
	WPU = 0b000011;              // pullups disabled sur GP0 et GP1
	
	GPIE = 0 ; 			// disable port change interrupt
	IOCB = 0b000000 ; 	// inhibe les IT sur ports en entre
	GIE = 0; 			// global interrupt disable

	

	
	
	i = 0;
	phase = 0;

	speed = 228;		           // 228 est la valeur du timer pour 4000 Hz

	

/* initialisation de la valeur max de chauffe */
// ==========================================

	          // max chauffe stockee a l'adresse 2 de l'EEPROM
      max = eeprom_read(2); 
	
	if ((max < 5)||(max > chauffe_max)) max = chauffe_max;    // filtre EEPROM non initialisee

                // max chauffe bougie 2 stockee a l'adresse 3 de l'EEPROM
      max2 = eeprom_read(3); 
	if ((max2 < 5)||(max2 > chauffe_max)) max2 = chauffe_max;    // filtre EEPROM non initialisee



/* lecture valeur chauffe bougie1  */
// ==============================
	          //  rapport chauffe bougie1 stockee a l'adresse 0 de l'EEPROM
      rapport1  = eeprom_read(0); 
	if (rapport1 > max) rapport1 = chauffe_min;    // filtre EEPROM non initialisee




/* lecture valeur chauffe bougie2  */
/* =============================== */

	      // rapport chauffe bougie2 stockee a l'adresse 1 de l'EEPROM
	
     rapport2 =  eeprom_read(1); 
	if (rapport2 > max2) rapport2 = chauffe_min;    // filtre EEPROM non initialisee



// resets au boostrap si BP PLUS appuye
// ====================================
	
	if (BOUGIE == bougie1)		// on est sur bougie 1
	{
	  if (PLUS == ON) 		// reset valeur max
	  {
		  max = chauffe_max;
		  			// stocke a l'adresse 2
		  tempo = max;
	   	  eeprom_write(2,tempo);	 // sauvegarde ds l'EEPROM
	  }
	}
	else // on est sur bougie 2
	{
	  if (PLUS == ON) 		// reset valeur max
	  {
		  max2 = chauffe_max;
		  			// stocke a l'adresse 3
		  tempo = max2;
	   	  eeprom_write(3,tempo);	 // sauvegarde ds l'EEPROM
	  }
	}


	
		
/* GESTION DES HORLOGES */
/* ==================== */
horloge :
	TMR0 = speed; 
	if (BOUGIE == bougie2) 		// inter ferme = bougie 2
	{
	phase++;          /* incremente phase de 0 a 200 a chaque demi periode 4000 Hz */
		if (phase < rapport2) 
		{
			S_CHAUFFE = 1;
			S_ECH = 1;  /* gestion sortie chauffe */
		}
		else 
		{
			S_CHAUFFE = 0 ;
			S_ECH = 0;
		}
		if (phase >= 200)		/* gestion des boutons */
		{
			phase = 0;
			if (PLUS == ON) 
			{
				if (i==0)		/* on peut valider */
				{
				  if (MOINS == ON) goto saut;
				  rapport2++ ; /* incremente rapport cyclique */
				  if (rapport2 > max2) rapport2 = max2;
				  
				  tempo = rapport2;
	   	 		  eeprom_write(1,tempo);	 // sauvegarde ds l'EEPROM
				  goto saut;
				}
				else if (i == 3)
				{
				  if (MOINS == ON)  // les 2 BP sont ON ensemble
						  // ==> memoriser valeur max chauffe
				  {
				    max2 = rapport2;
				    
				    tempo = max2;
	   	 		    eeprom_write(3,tempo);	 // sauvegarde ds l'EEPROM
				    goto saut;
				  }
				}
			}
 			if (MOINS == ON) 
			{
				if (i==0)		/* on peut valider */
				{
				  rapport2-- ; /* decremente rapport cyclique */
				  if (rapport2 == chauffe_min-1) rapport2 = chauffe_min;
				  
	   	 		  tempo = rapport2;
	   	 		  eeprom_write(1,tempo);	 // sauvegarde ds l'EEPROM
				}
			}
saut :
			i++;				/* filtre  environ 0,37 sec */
			if (i==15) i=0;
		}
	}

	else			// inter ouvert = bougie 1
	{
					
		phase++;          /* incremente phase de 0 a 200*/
		if (phase < rapport1) 
		{
			S_CHAUFFE = 1;
			S_ECH = 1;  /* gestion sortie chauffe */
		}
		else 
		{
			S_CHAUFFE = 0 ;
			S_ECH = 0;
		}
		if (phase >= 200)		/* gestion des boutons */
		{
			phase = 0;
			if (PLUS == ON) 
			{
				if (i==0)		/* on peut valider */
				{
				  if (MOINS == ON) goto saut;
				  rapport1++ ; /* incremente rapport cyclique */
				  if (rapport1 > max) rapport1 = max;
				 
				  tempo = rapport1;
	   	 		  eeprom_write(0,tempo);	 // sauvegarde ds l'EEPROM
				  goto saut2;
				}
				else if (i == 3)
				{
				  if (MOINS == ON)  // les 2 BP sont ON ensemble
						  // ==> memoriser valeur max chauffe
				  {
				    max = rapport1;
				   
				    tempo = max;
	   	 		    eeprom_write(2,tempo);	 // sauvegarde ds l'EEPROM
				    goto saut2;
				  }
				}
			}
 			if (MOINS == ON) 
			{
				if (i==0)		/* on peut valider */
				{
				  rapport1-- ; /* decremente rapport cyclique */
				  if (rapport1 == chauffe_min-1) rapport1 = chauffe_min;
				 
	   	 		  tempo = rapport1;
	   	 		  eeprom_write(0,tempo);	 // sauvegarde ds l'EEPROM
				}
			}
saut2 :
			i++;				/* filtre  environ 0,37 sec */
			if (i==15) i=0;
		}
	}
	do {			/* attente fin du timer de 250 - 6 usec */
	} while (TMR0);
	S_HORL = !S_HORL;
	asm("NOP");
goto horloge ;
}

/*******************interrupt service routines*****************/
void interrupt isr(void)
{
	
	GIE = 0;                //disable interrupts & return main	
}
/*******************************the end **************************/
